/* First, the standard lib includes, alphabetically ordered */
#include <assert.h>
#include <dirent.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* Then, this project's includes, alphabetically ordered */
#include "array_helpers.h"
#include "sort.h"

#define MAX_SIZE 10000

/* Helpers to print comparison data for every sorting algorithm */
typedef struct _sorting_func_t {
    const char *name;
    unsigned int (*sorting_func) (int a[], unsigned int length);
} sorting_func_t;

static sorting_func_t sorting_func_table[] = {
    /* The basic sorting algorithms */
    {"Selection sort", selection_sort},
    {"Insertion sort", insertion_sort},
    {"Quick sort", quick_sort},
    {"Rand Quick sort", rand_quick_sort}, 

    /* Do not remove the following, it indicates the array end */
    {NULL, NULL},
};

static bool is_array_file(const char *filename) {
    unsigned int filename_len = strlen(filename);

    assert(filename != NULL);

    return (filename_len > 3 &&
            strcmp(filename + filename_len - 3, ".in") == 0);
}

static char *join_dirame_filename(const char *dirname, const char *filename) {
    char *filepath = NULL;

    assert(dirname != NULL);
    assert(filename != NULL);

    filepath = calloc(strlen(dirname) + strlen(filename) + 2, sizeof(char));
    assert(filepath != NULL);

    filepath = strncat(filepath, dirname, strlen(dirname));
    filepath = strncat(filepath, "/", 1);
    filepath = strncat(filepath, filename, strlen(filename));

    return (filepath);
}

static void process_array_file(const char *filename) {
    unsigned int i = 0, length = 0;
    int array[MAX_SIZE];
    int copy[MAX_SIZE];

    printf(" %-28s | %5s\n", filename, "Comps");

    length = array_from_file(array, MAX_SIZE, filename);

    while (sorting_func_table[i].name != NULL) {
        array_copy(copy, array, length);
        unsigned int cmp = sorting_func_table[i].sorting_func(copy, length);

        assert(array_is_sorted(copy, length));
        assert(array_is_permutation_of(array, copy, length));

        printf(" %-28s | %u\n",
               sorting_func_table[i].name, cmp);
        i++;
    }
    printf("\n");
}

int main(int argc, char *argv[]) {
    DIR *dp = NULL;
    struct dirent *ep = NULL;
    char *dirname = NULL;

    if (argc < 2) {
        printf("Need a directory path to scan for array files.\n");
        exit(EXIT_FAILURE);
    }

    dirname = argv[1];
    dp = opendir(dirname);
    if (dp == NULL) {
        perror("Couldn't open the directory");
        exit(EXIT_FAILURE);
    }

    ep = readdir(dp);
    while (ep != NULL) {
        char *filename = ep->d_name;
        if (is_array_file(filename)) {
            /* build path to array file concatening dirname and filename */
            char *filepath = join_dirame_filename(dirname, filename);
            process_array_file(filepath);
            free(filepath);
        }
        ep = readdir(dp);
    }
    closedir(dp);
    return (EXIT_SUCCESS);
}
